@@ -14,6 +14,7 @@ import locale
 import time
 from xml.etree import ElementTree
 from PIL import Image
+import hashlib
 
 gettext.install("cinnamon", "/usr/share/locale")
 
@@ -205,6 +206,8 @@ class Module:
             widget = GSettingsColorChooser(_("Gradient end color"), "org.cinnamon.desktop.background", "secondary-color", size_group=size_group)
             self.secondary_color_revealer = settings.add_reveal_row(widget)
 
+            self.update_secondary_revealer(self._background_schema, None)
+
     def is_row_separator(self, model, iter, data):
         return model.get_value(iter, 0)
 
@@ -503,29 +506,33 @@ class PixCache(object):
             pix = self._data[filename][size]
         else:
             try:
-                if mimetype == "image/svg+xml":
-                    tmp_pix = GdkPixbuf.Pixbuf.new_from_file(filename)
-                    tmp_fp, tmp_filename = tempfile.mkstemp()
-                    os.close(tmp_fp)
-                    tmp_pix.savev(tmp_filename, "png", [], [])
-                    img = Image.open(tmp_filename)
-                    os.unlink(tmp_filename)
+                h = hashlib.sha1(('%f%s' % (os.path.getmtime(filename), filename)).encode()).hexdigest()
+                tmp_cache_path = GLib.get_user_cache_dir() + '/cs_backgrounds/'
+                if not os.path.exists(tmp_cache_path):
+                    os.mkdir(tmp_cache_path)
+                cache_filename = tmp_cache_path + h
+                if os.path.exists(cache_filename):
+                    (width, height) = Image.open(filename).size
                 else:
-                    img = Image.open(filename)
-                (width, height) = img.size
-                if img.mode != 'RGB':
-                    img = img.convert('RGB')
-                if size:
-                    img.thumbnail((size, size), Image.ANTIALIAS)
-                img = imtools.round_image(img, {}, False, None, 3, 255)
-                img = imtools.drop_shadow(img, 4, 4, background_color=(255, 255, 255, 0), shadow_color=0x444444, border=8, shadow_blur=3, force_background_color=False, cache=None)
-                # Convert Image -> Pixbuf (save to file, GTK3 is not reliable for that)
-                f = tempfile.NamedTemporaryFile(delete=False)
-                temp_filename = f.name
-                f.close()
-                img.save(temp_filename, "png")
-                pix = [GdkPixbuf.Pixbuf.new_from_file(temp_filename), width, height]
-                os.unlink(temp_filename)
+                    if mimetype == "image/svg+xml":
+                        tmp_pix = GdkPixbuf.Pixbuf.new_from_file(filename)
+                        tmp_fp, tmp_filename = tempfile.mkstemp()
+                        os.close(tmp_fp)
+                        tmp_pix.savev(tmp_filename, "png", [], [])
+                        img = Image.open(tmp_filename)
+                        os.unlink(tmp_filename)
+                    else:
+                        img = Image.open(filename)
+                    (width, height) = img.size
+                    if img.mode != 'RGB':
+                        img = img.convert('RGB')
+                    if size:
+                        img.thumbnail((size, size), Image.ANTIALIAS)
+                    img = imtools.round_image(img, {}, False, None, 3, 255)
+                    img = imtools.drop_shadow(img, 4, 4, background_color=(255, 255, 255, 0), shadow_color=0x444444, border=8, shadow_blur=3, force_background_color=False, cache=None)
+                    # Convert Image -> Pixbuf (save to file, GTK3 is not reliable for that)
+                    img.save(cache_filename, "png")
+                pix = [GdkPixbuf.Pixbuf.new_from_file(cache_filename), width, height]
             except Exception, detail:
                 print "Failed to convert %s: %s" % (filename, detail)
                 pix = None
@@ -654,7 +661,7 @@ class ThreadedIconView(Gtk.IconView):
                     dimensions = "%dx%d" % (pix[1], pix[2])
 
                     self._loaded_data_lock.acquire()
-                    self._loaded_data.append((to_load, pix[0], "<b>%s</b>\n<sub>%s<span foreground='#555555'>%s</span></sub>" % (label, artist, dimensions), path))
+                    self._loaded_data.append((to_load, pix[0], "<b>%s</b>\n<sub>%s%s</sub>" % (label, artist, dimensions), path))
                     self._loaded_data_lock.release()
 
         self._loading_lock.acquire()
@@ -671,6 +678,8 @@ class ThreadedIconView(Gtk.IconView):
                     if backgroundNode.tag == "static":
                         for staticNode in backgroundNode:
                             if staticNode.tag == "file":
+                                if staticNode[-1].tag == "size":
+                                    return staticNode[-1].text
                                 return staticNode.text
             print "Could not find filename in %s" % filename
             return None
