@@ -440,6 +440,7 @@ Notification.prototype = {
         this._bannerBodyText = null;
         this._bannerBodyMarkup = false;
         this._titleFitsInBannerMode = true;
+        this._inhibitTransparency = false;
         this._titleDirection = St.TextDirection.NONE;
         this._spacing = 0;
 
@@ -460,24 +461,8 @@ Notification.prototype = {
         this.actor._parent_container = null;
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
-		// Transparency on mouse over?
-		if (Main.messageTray.fadeOnMouseover) {
-			// Register to every notification as we intend to support multiple notifications on screen.
-			this.enter_id = this.actor.connect('enter-event', Lang.bind(this, function() {
-				Tweener.addTween(this.actor, {
-					opacity: ((Main.messageTray.fadeOpacity / 100) * 255).clamp(0, 255),
-					time: ANIMATION_TIME,
-					transition: 'easeOutQuad'
-				});
-			}));
-			this.leave_id = this.actor.connect('leave-event', Lang.bind(this, function() {
-				Tweener.addTween(this.actor, {
-					opacity: (this._table.get_theme_node().get_length('opacity') / global.ui_scale) || 255,
-					time: ANIMATION_TIME,
-					transition: 'easeOutQuad'
-				});
-			}));
-		}
+
+        this.updateFadeOnMouseover();
 
         this._table = new St.Table({ name: 'notification',
                                      reactive: true });
@@ -623,6 +608,36 @@ Notification.prototype = {
         this._updated();
     },
 
+    updateFadeOnMouseover: function() {
+        // Transparency on mouse over?
+        if (Main.messageTray.fadeOnMouseover && !this._inhibitTransparency) {
+            // Register to every notification as we intend to support multiple notifications on screen.
+            this.enter_id = this.actor.connect('enter-event', Lang.bind(this, function() {
+                Tweener.addTween(this.actor, {
+                    opacity: ((Main.messageTray.fadeOpacity / 100) * 255).clamp(0, 255),
+                    time: ANIMATION_TIME,
+                    transition: 'easeOutQuad'
+                });
+            }));
+            this.leave_id = this.actor.connect('leave-event', Lang.bind(this, function() {
+                Tweener.addTween(this.actor, {
+                    opacity: (this._table.get_theme_node().get_length('opacity') / global.ui_scale) || 255,
+                    time: ANIMATION_TIME,
+                    transition: 'easeOutQuad'
+                });
+            }));
+        } else {
+            if (this.enter_id > 0) {
+                this.actor.disconnect(this.enter_id);
+                this.enter_id = 0;
+            }
+            if (this.leave_id > 0) {
+                this.actor.disconnect(this.leave_id);
+                this.leave_id = 0;
+            }
+        }
+    },
+
     setIconVisible: function(visible) {
         this._icon.visible = visible;
     },
@@ -798,6 +813,10 @@ Notification.prototype = {
         this._buttonFocusManager.add_group(this._buttonBox);
         button.connect('clicked', Lang.bind(this, this._onActionInvoked, id));
 
+        this._inhibitTransparency = true;
+
+        this.updateFadeOnMouseover();
+
         this._updated();
     },
 
