@@ -73,6 +73,7 @@ struct _CinnamonApp
   char *name_collation_key;
   char *casefolded_description;
   char *casefolded_exec;
+  char *keywords;
 };
 
 G_DEFINE_TYPE (CinnamonApp, cinnamon_app, G_TYPE_OBJECT);
@@ -254,7 +255,7 @@ cinnamon_app_create_faded_icon_cpu (StTextureCache *cache,
     return COGL_INVALID_HANDLE;
 
   pixbuf = gtk_icon_info_load_icon (info, NULL);
-  gtk_icon_info_free (info);
+  g_object_unref (info);
 
   if (pixbuf == NULL)
     return COGL_INVALID_HANDLE;
@@ -385,6 +386,40 @@ cinnamon_app_get_description (CinnamonApp *app)
     return NULL;
 }
 
+const char *
+cinnamon_app_get_keywords (CinnamonApp *app)
+{
+  const char * const *keywords;
+  const char *keyword;
+  gint i;
+  gchar *ret = NULL;
+
+  if (app->keywords)
+    return app->keywords;
+
+  if (app->entry)
+    keywords = g_desktop_app_info_get_keywords (G_DESKTOP_APP_INFO (gmenu_tree_entry_get_app_info (app->entry)));
+  else
+    keywords = NULL;
+
+  if (keywords != NULL)
+    {
+      GString *keyword_list = g_string_new(NULL);
+
+      for (i = 0; keywords[i] != NULL; i++)
+        {
+          keyword = keywords[i];
+          g_string_append_printf (keyword_list, "%s;", keyword);
+        }
+
+      ret = g_string_free (keyword_list, FALSE);
+    }
+
+    app->keywords = ret;
+
+    return ret;
+}
+
 /**
  * cinnamon_app_is_window_backed:
  *
@@ -1353,6 +1388,7 @@ static void
 cinnamon_app_init (CinnamonApp *self)
 {
   self->state = CINNAMON_APP_STATE_STOPPED;
+  self->keywords = NULL;
 }
 
 static void
@@ -1372,6 +1408,8 @@ cinnamon_app_dispose (GObject *object)
         _cinnamon_app_remove_window (app, app->running_state->windows->data);
     }
 
+  g_clear_pointer (&app->keywords, g_free);
+
   G_OBJECT_CLASS(cinnamon_app_parent_class)->dispose (object);
 }
 
